/**
 * @file main.c
 * @brief Leds verde e vermelho piscante
 * @author Wu Shin-Ting
 * @date 06/01/2023
 */

//Inclusao dos tipos de dados padronizados pelo ISO C99
#include "stdint.h"

/**
 * @brief Conffiguracao especifica dos modulos 
 */
void config_especifica() {
	/**
	 * Configura o modulo GPIOE: sentido de saida do sinal no pino PTE21
	 * registrador GPIOE_PDDR mapeado em 0x400ff114u
	 */
	(*(uint32_t volatile *) 0x400ff114u) |= (1<<21);
	
	/**
	 * Configura o modulo GPIOE: sentido de saida do sinal no pino PTE22
	 */
	(*(uint32_t volatile *) 0x400ff114u) |= (1<<22);
	
	/**
	 * Configura o modulo GPIOA: sentido de entrada do sinal no pino PTA4, sem
	 * interrupcao habilitada
	 * registrador GPIOA_PDDR mapeado em 0x400ff014u
	 */
	(*(uint32_t volatile *) 0x400ff014u) &= ~(1<<4);
			
	/**
	 * Configura o modulo GPIOA: sentido de entrada do sinal no pino PTA5, com interrupcao
	 * habilitada
	 * registrador GPIOA_PDDR mapeado em 0x400ff014u
	 */
	(*(uint32_t volatile *) 0x400ff014u) &= ~(1<<5);	

	/**
	 * Limpa flag de interrupcao do pino 5
	 * registrador PORTA_PCR5 mapeado em 0x40049014u 
	 */
	(*(uint32_t volatile *) 0x40049014u) |= 0x1000000u; //0x1000000u = (1<<24)
			
	/**
	 * Habilita a interrupcao do pino 5 para ambas as bordas
	 */
	*(uint32_t volatile *) 0x40049014u &= ~0xF0000; 
	(*(uint32_t volatile *) 0x40049014u) |= 0x000B0000; 

	/**
	 * Configura o modulo NVIC: habilita IRQ 30
	 * registrador NVIC_ISER mapeado em 0xe000e100u
	 */
	(*(uint32_t volatile *) 0xe000e100u) = (1<<30);	

	/**
	 * Configura o modulo NVIC: limpa pendencias IRQ 30
	 * registrador NVIC_ICPR mapeado em 0xe000e280u
	 */
	(*(uint32_t volatile *) 0xe000e280u) = (1<<30);	

	/**
	 * Configura o modulo NVIC: seta prioriodade 3 para IRQ30
	 * registrador NVIC_IPR7 mapeado em 0xe000e41cu (Tabela 3-7/p. 54 do Manual)
	 */
	(*(uint32_t volatile *) 0xe000e41cu) |= (0b11<<22); 	//(Secao 3.3.2.1/p. 52 do Manual)
}

/**
 * @brief Configuracao basica de operacao dos modulos
 */
void config_basica () {
	/**
	 * Configura o modulo SIM: ativacao dos sinais de relogio de PORTE
	 * Ativacao dos sinais de relogio de PORTE
	 */
	*(uint32_t volatile *) 0x40048038u |= (1<<13);
	/**
	 * Ativacao dos sinais de relogio de PORTA
	 */
	*(uint32_t volatile *) 0x40048038u |= (1<<9);

	/**
	 * Configura o modulo PORTE_PCR21
	 * registrador PORTE_PCR21 mapeado em 0x4004d054 
	 */
	*(uint32_t volatile *) 0x4004d054u &= ~0x700; // Zera bits 10, 9 e 8 (MUX) de PTE21
	(*(uint32_t volatile *) 0x4004d054u) |= 0x00000100; // Seta bit 8 do MUX de PTE21
	
	/**
	 * Configura o modulo PORTE_PCR22
	 * registrador PORTE_PCR22 mapeado em 0x4004d058 
	 */
	*(uint32_t volatile *) 0x4004d058u &= ~0x700; // Zera bits 10, 9 e 8 (MUX) de PTE22
	(*(uint32_t volatile *) 0x4004d058u) |= 0x00000100; // Seta bit 8 do MUX de PTE22
	
	/**
	 * Configura o modulo PORTA_PCR4
	 * registrador PORTA_PCR4 mapeado em 0x40049010 
	 */
	*(uint32_t volatile *) 0x40049010u &= ~0x700; // Zera bits 10, 9 e 8 (MUX) de PTA4
	(*(uint32_t volatile *) 0x40049010u) |= 0x00000100; // Seta bit 8 do MUX de PTA4

	/**
	 * Configura o modulo PORTA_PCR5
	 * registrador PORTA_PCR5 mapeado em 0x40049014 
	 */
	*(uint32_t volatile *) 0x40049014u &= ~0x700; // Zera bits 10, 9 e 8 (MUX) de PTA5
	(*(uint32_t volatile *) 0x40049014u) |= 0x00000100; // Seta bit 8 do MUX de PTBA5
}

/**
 * @inicializacao do estado do sistema
 */
void inicializacao () {
	/**
	 * Inicializa o estado PTE21 em 1 (LED ativo alto)
	 * registrador GPIOE_PSOR mapeado em 0x400ff104u 
	 */
	(*(uint32_t volatile *) 0x400ff104u) = (1<<21);

	/**
	 * Inicializa o estado PTE22 em 1 (LED ativo alto)
	 */
	(*(uint32_t volatile *) 0x400ff104u) = (1<<22);
}

/**
 * @definicao da rotina de servico declarada
 */
void PORTA_IRQHandler (){
	/**
	 * Busca o bit de estado correspondente
	 */
	if ((*(uint32_t volatile *) 0x40049014u) & 0x1000000) {
		/**
		 * Verifica o valor do bit PTA5
		 */
		if ((*(uint32_t volatile *) 0x400ff010u) & 0x20) {
			// 1 = ocorreu uma borda de subida
			// Chave desativada (PTE22 em 1)
			(*(uint32_t volatile *)0x400ff104u) = (1 << 22);			
		} else {
			// 0 = ocorreu uma borda de descida
			// Chave ativada (PTE22 em 0)
			(*(uint32_t volatile *)0x400ff108u) = (1 << 22);						
		}
		/**
		 * Limpa flag 
		 */
		(*(uint32_t volatile *) 0x40049014u) |= (1<<24);
	}
}
/**
 * @brief Fluxo de controle principal
 */
int main(void)
{
	config_basica();

	config_especifica();

	inicializacao();

	for (;;) {
		if (!(*((uint32_t volatile *)0x400ff010u) & (1<<4))) {
			// Chave ativa baixa (PTE21 em 0)
			(*(uint32_t volatile *)0x400ff108u) = (1 << 21);

			while (!(*((uint32_t volatile *)0x400ff010u) & 0x10)); //aguarda enquanto fique em 0

			// Chave desativada (PTE21 em 1)
			(*(uint32_t volatile *)0x400ff104u) = (1 << 21);
		}
	}

	/* Never leave main */
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// EOF
////////////////////////////////////////////////////////////////////////////////
